<?php
/*
Plugin Name: Calendar Press
Plugin URI: http://calendarpress.net/
Description: Add an event calendar with details, Google Maps, directions, RSVP system and more.
Author: GrandSlambert
Version: 0.2.0
Author: GrandSlambert
Author URI: http://www.grandslambert.com/

**************************************************************************

Copyright (C) 2010 GrandSlambert

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************

*/

require_once('includes/initialize.php');
require_once('includes/template_tags.php');
require_once('includes/inflector.php');

if (!function_exists('recaptcha_get_html')) {
    require_once('includes/recaptchalib.php');
}

class calendarPress {

    public $menuName = 'calendar-press';
    public $pluginName = 'Calendar Press';
    public $version = '0.2.0';
    public $optionsName	= 'calendar-press-options';
    protected $xmlURL = 'http://grandslambert.com/xml/calendar-press/';

    /**
     * Initialize the plugin.
     */
    public function __construct() {
        /* Load Langague Files */
        $langDir = dirname( plugin_basename(__FILE__) ) . '/lang';
        load_plugin_textdomain( 'calendar-press', false, $langDir, $langDir );

        /* Plugin Settings */
        /* translators: The name of the plugin, should be a translation of "Calendar Press" only! */
        $this->pluginName = __('Calendar Press', 'calendar-press');
        $this->pluginPath = WP_CONTENT_DIR . '/plugins/' . basename(dirname(__FILE__));
        $this->pluginURL = get_option('siteurl') . '/wp-content/plugins/' . basename(dirname(__FILE__));
        $this->templatesPath = WP_CONTENT_DIR . '/plugins/' . basename(dirname(__FILE__)) . '/templates/';
        $this->templatesURL = get_option('siteurl') . '/wp-content/plugins/calendar-press/templates/';
        $this->loadSettings();

        /* Add Options Pages and Links */
        add_action('wp_head', array($this, 'addHeader') );
        add_action('admin_menu', array(&$this, 'addAdminPages'));
        add_filter('plugin_action_links', array(&$this, 'addConfigureLink'), 10, 2);
        add_action('admin_head', array($this, 'adminHeader') );
        add_filter('whitelist_options', array(&$this, 'whitelistOptions'));
        add_action('admin_init', array(&$this, 'registerOptions'));
        add_action('save_post', array(&$this, 'save_event'));
        add_filter('the_content', array(&$this, 'the_content_filter'));
        add_filter('the_excerpt', array(&$this, 'the_content_filter'));
        add_action('right_now_content_table_end', array(&$this, 'right_now_content_table_end'));
        add_filter('query_vars', array(&$this, 'add_queryvars') );
        add_action('pre_get_posts', array(&$this, 'pre_get_posts'));

        /* Setup AJAX */
        add_action('wp_ajax_event_registration', array(&$this, 'event_registration'));
        add_action('wp_ajax_nopriv_event_registration', array(&$this, 'event_registration'));

        $init = new calendarPress_init($this->options);
    }

    /**
     * Add the number of events to the Right Now on the Dasboard.
     */
    public function right_now_content_table_end() {
        if (!post_type_exists('events')) {
            return false;
        }

        $num_posts = wp_count_posts( 'events' );
        $num = number_format_i18n( $num_posts->publish );
        $text = _n( 'Events', 'Events', intval($num_posts->publish) );
        if ( current_user_can( 'edit_posts' ) ) {
            $num = "<a href='edit.php?post_type=events'>$num</a>";
            $text = "<a href='edit.php?post_type=events'>$text</a>";
        }
        echo '<td class="first b b-events">' . $num . '</td>';
        echo '<td class="t events">' . $text . '</td>';

        echo '</tr>';

        if ($num_posts->pending > 0) {
            $num = number_format_i18n( $num_posts->pending );
            $text = _n( 'Events Pending', 'Events Pending', intval($num_posts->pending) );
            if ( current_user_can( 'edit_posts' ) ) {
                $num = "<a href='edit.php?post_status=pending&post_type=events'>$num</a>";
                $text = "<a href='edit.php?post_status=pending&post_type=events'>$text</a>";
            }
            echo '<td class="first b b-events">' . $num . '</td>';
            echo '<td class="t events">' . $text . '</td>';

            echo '</tr>';
        }
    }

    /**
     * Load plugin settings.
     */
    public function loadSettings() {
        if (!$this->options = get_option($this->optionsName) ) {
            $this->options['use-signups'] = true;
            $this->options['use-waiting'] = true;
            $this->options['use-categories'] = true;
            $this->options['use-tags'] = true;
            $this->options['use-thumbnails'] = true;
            $this->options['use-featured'] = true;
            $this->options['use-locations'] = true;
            $this->options['submit-title'] = __('Submit An Event', 'calendar-press');
            $this->options['widget-items'] = 10;
            $this->options['widget-icon-size'] = 25;
            $this->options['new-event-status'] = 'pending';
            $this->options['slug'] = 'events';

            $this->options['signups-title'] = 'Openings';
            $this->options['overflow-title'] = 'Overflow';
            $this->options['waiting-title'] = 'Waiting List';
            $this->options['default-excerpt-length'] = 20;
            $this->options['slug'] = 'events';
        }

        /* Non-Configurable Settings */
        $this->options['menu-icon'] = $this->pluginURL . '/images/icons/small_icon.png';
        $this->options['widget-before-count'] = ' ( ';
        $this->options['widget-after-count'] = ' ) ';


        $this->options['states'] = array(
            'AL' => 'Alabama',
            'AK' => 'Alaska',
            'AZ' => 'Arizona',
            'AR' => 'Arkansas',
            'CA' => 'California',
            'CO' => 'Colorado',
            'CT' => 'Connecticut',
            'DE' => 'Delaware',
            'DC' => 'District of Columbia',
            'FL' => 'Florida',
            'GA' => 'Georgia',
            'HI' => 'Hawaii',
            'ID' => 'Idaho',
            'IL' => 'Illinois',
            'IN' => 'Indiana',
            'IA' => 'Iowa',
            'KS' => 'Kansas',
            'KY' => 'Kentucky',
            'LA' => 'Louisiana',
            'ME' => 'Maine',
            'MD' => 'Maryland',
            'MA' => 'Massachusetts',
            'MI' => 'Michigan',
            'MN' => 'Minnesota',
            'MS' => 'Mississippi',
            'MO' => 'Missouri',
            'MT' => 'Montana',
            'NE' => 'Nebraska',
            'NV' => 'Nevada',
            'NH' => 'New Hampshire',
            'NJ' => 'New Jersey',
            'NM' => 'New Mexico',
            'NY' => 'New York',
            'NC' => 'North Carolina',
            'ND' => 'North Dakota',
            'OH' => 'Ohio',
            'OK' => 'Oklahoma',
            'OR' => 'Oregon',
            'PA' => 'Pennsylvania',
            'RI' => 'Rhode Island',
            'SC' => 'South Carolina',
            'SD' => 'South Dakota',
            'TN' => 'Tennessee',
            'TX' => 'Texas',
            'UT' => 'Utah',
            'VT' => 'Vermont',
            'VA' => 'Virginia',
            'WA' => 'Washington',
            'WV' => 'West Virginia',
            'WI' => 'Wisconsin',
            'WY' => 'Wyoming',
        );

        /* Required settings */
        if (!$this->options['slug']) {
            $this->options['slug'] = 'events';
        }

        if (!$this->options['signups-title']) {
            $this->options['signups-title'] = 'Openings';
        }

        if (!$this->options['overflow-title']) {
            $this->options['overflow-title'] = 'Overflow';
        }

        if (!$this->options['waiting-title']) {
            $this->options['waiting-title'] = 'Waiting List';
        }

        if (!$this->options['box-width']) {
            $this->options['box-width'] = '85';
        }

        if (!$this->options['user-display-field']) {
            $this->options['user-display-field'] = 'display_name';
        }
    }

    /**
     * Add items to the header of the web site.
     */
    public function addHeader() {
        echo '<link rel="stylesheet" media="screen" type="text/css" href="' . $this->getTemplate('calendar-press', '.css', 'url') .'" />' . "\n";
        ?>
<style type="text/css" media="screen">
    .cp-box-width, dd.cp-month-box {
        width: <?php echo $this->options['box-width']; ?>px;
    }
</style>
        <?php
        echo '<script type="text/javascript" src="' . $this->pluginURL . '/js/calendar-press.js"></script>' . "\n";
        echo '<script type="text/javascript" src="' . $this->pluginURL . '/js/overlib/overlib.js"></script>' . "\n";
        wp_print_scripts( array( 'sack' ));
    }

    /**
     * Add necessary code to the head section for the administration pages.
     */
    public function adminHeader() {
        global $post;
        if ($post->post_type == 'events' OR preg_match('/calendar-press/', $_SERVER['REQUEST_URI'])) {
            echo '<link rel="stylesheet" media="screen" type="text/css" href="' . $this->getTemplate('calendar-press-admin', '.css', 'url'). '" />' . "\n";
            echo '<link rel="stylesheet" media="screen" type="text/css" href="' . $this->getTemplate('datepicker', '.css', 'url'). '" />' . "\n";
            echo '<script type="text/javascript" src="' . $this->pluginURL . '/js/overlib/overlib.js"></script>' . "\n";
            echo '<script type="text/javascript" src="' . $this->pluginURL . '/js/datepicker/js/datepicker.js"></script>' . "\n";
            echo '<script type="text/javascript" src="' . $this->pluginURL . '/js/calendar-press-admin.js"></script>' . "\n";
        }
    }

    public function add_queryvars( $qvars ) {
        $qvars[] = 'viewmonth';
        $qvars[] = 'viewyear';
        return $qvars;
    }

    /**
     * Add settings vars to the whitelist for forms.
     *
     * @param array $whitelist
     * @return array
     */
    function whitelistOptions($whitelist) {
        if (is_array($whitelist)) {
            $option_array = array($this->pluginName => $this->optionsName);
            $whitelist = array_merge($whitelist, $option_array);
        }

        return $whitelist;
    }

    /**
     * Add the admin page for the settings panel.
     *
     * @global string $wp_version
     */
    function addAdminPages() {
        global $wp_version, $wpdb;

        add_submenu_page('edit.php?post_type=events', 'Settings', 'Settings', 8, $this->menuName, array(&$this, 'settings'));
        add_submenu_page('edit.php?post_type=events', 'Contributors', 'Contributors', 8, 'calendar-press-contributors', array(&$this, 'contributors'));

        $tableName = $wpdb->prefix . 'rp_events';
        if ($wpdb->get_var("SHOW TABLES LIKE '{$tableName}'") == $tableName) {
            add_submenu_page('edit.php?post_type=events', 'Convert', 'Convert Events', 8, 'calendar-press-convert', array(&$this, 'convert'));
        }

        // Use the bundled jquery library if we are running WP 2.5 or above
        if (version_compare($wp_version, '2.5', '>=')) {
            wp_enqueue_script('jquery', false, false, '1.2.3');
        }
    }

    /**
     * Add a configuration link to the plugins list.
     *
     * @staticvar <object> $this_plugin
     * @param <array> $links
     * @param <array> $file
     * @return <array>
     */
    function addConfigureLink($links, $file) {
        static $this_plugin;

        if (!$this_plugin) {
            $this_plugin = plugin_basename(__FILE__);
        }

        if ($file == $this_plugin) {
            $settings_link = '<a href="' . get_admin_url() . 'edit.php?post_type=events&page=' . $this->menuName . '">' . __('Settings', 'calendar-press') . '</a>';
            array_unshift($links, $settings_link);
        }

        return $links;
    }

    /**
     * Settings management panel.
     */
    function settings() {
        include($this->pluginPath . '/includes/settings.php');
    }

    /**
     * Settings management panel.
     */
    function contributors() {
        include($this->pluginPath . '/includes/contributors.php');
    }

    /**
     * Settings management panel.
     */
    function convert() {
        if (!$_POST) {
            include($this->pluginPath . '/includes/convert.php');
            return;
        }

        require($this->pluginPath . '/includes/converter.php');
    }

    /**
     * Create a help icon on the administration pages.
     *
     * @param <string> $text
     */
    public function help($text) {
        echo '<img src="' . $this->pluginURL . '/images/icons/help.jpg" align="absmiddle"onmouseover="return overlib(\'' . $text . '\');" onmouseout="return nd();" />';
    }

    /**
     * Register the options
     */
    function registerOptions() {
        register_setting( $this->optionsName, $this->optionsName);
    }

    function pre_get_posts() {
        global $wp_query;

        if (is_archive() and array_key_exists('event-categories', $wp_query->query_vars)) {
            $wp_query->query_vars['orderby'] = 'post_date';
            $wp_query->query_vars['order'] = 'ASC';
        }
    }

    /**
     * Save the meta boxes for a event.
     *
     * @global <object> $post
     * @param <integer> $post_id
     * @return <integer>
     */
    function save_event($post_id) {
        global $post;

        if ( wp_verify_nonce( $_POST['dates_noncename'], 'calendar_press_dates' )) {
            $details = $_POST['event_dates'];

            foreach ($details as $key=>$value) {

                list($type,$field) = split('_', $key);

                if ($field == 'time') {
                    $meridiem = $_POST[$type . 'Meridiem'];
                    $minutes = $_POST[$type . '_time_minutes'];
                    $value = $value + $meridiem;
                    $value = $value . ':' . $minutes;

                }

                if ($key == 'begin_date') {
                    $beginDate = $value;
                }

                if ($key == 'begin_time') {
                    $beginTime = $value;
                }

                $key = '_' . $key . '_value';
                $value = strtotime($value);

                if(get_post_meta($post_id, $key) == "") {
                    add_post_meta($post_id, $key, $value, true);
                } elseif($value != get_post_meta($post_id, $key.'_value', true)) {
                    update_post_meta($post_id, $key, $value);
                } elseif($value == "") {
                    delete_post_meta($post_id, $key, get_post_meta($post_id, $key, true));
                }
            }
        }


        $post->post_date = date ('Y-m-d H:i:s', strtotime($beginDate . ' ' . $beginTime));
        remove_action('save_post', array(&$this, 'save_event'));
        wp_update_post($post);
        add_action('save_post', array(&$this, 'save_event'));

        if ( wp_verify_nonce( $_POST['details_noncename'], 'calendar_press_details' )) {
            $details = $_POST['event_details'];

            if (is_array($details)) {
                foreach ($details as $key=>$value) {

                    $key = '_' . $key . '_value';
                    if(get_post_meta($post_id, $key) == "") {
                        add_post_meta($post_id, $key, $value, true);
                    } elseif($value != get_post_meta($post_id, $key.'_value', true)) {
                        update_post_meta($post_id, $key, $value);
                    } elseif($value == "") {
                        delete_post_meta($post_id, $key, get_post_meta($post_id, $key, true));
                    }
                }
            }

            if (!$_POST['event_details']['event_featured']) {
                delete_post_meta($post_id, '_event_featured_value', get_post_meta($post_id,'_event_featured_value', true));

            }
        }

        global $wp_rewrite;
        $wp_rewrite->flush_rules();

        return $post_id;
    }

    public function the_content_filter($content) {
        global $post, $wp;

        if ($post->post_type != 'events' and $post->ID != $this->options['calendar-page']) {
            return $content;
        }

        remove_filter('the_content', array(&$this, 'the_content_filter'));

        if (is_archive()) {
            $template = $this->getTemplate('loop-event');
        } elseif (is_single() ) {
            $template = $this->getTemplate('single-event');
        } elseif ($post->post_type == 'events' and in_the_loop() ) {
            $template = $this->getTemplate('loop-event');
        } elseif ($post->ID == $this->options['calendar-page']) {
            $template = $this->getTemplate('index-event');

            /* Get the months */
            if (!$this->currMonth = $wp->query_vars['viewmonth']) {
                $this->currMonth = date('m');
            }
            if (!$this->currYear = $wp->query_vars['viewyear']) {
                $this->currYear = date('Y');
            }

            $lastmonth = $this->currMonth - 1;
            $nextmonth = $this->currMonth + 1;
            $this->currDate = date('Y-m-d', strtotime($this->currYear . '-' . $this->currMonth . '-' . date('d')));
            $this->lastMonth = date('Y-m-d', strtotime($this->currYear . '-' . $lastmonth . '-' . date('d')));
            $this->nextMonth = date('Y-m-d', strtotime($this->currYear . '-' . $nextmonth . '-' . date('d')));
        }

        ob_start();
        require ($template);
        $content = ob_get_contents();
        ob_end_clean();

        add_filter('the_content', array(&$this, 'the_content_filter'));

        return $content;
    }

    /**
     * Retrieve a template file from either the theme or the plugin directory.
     *
     * @param <string> $template    The name of the template.
     * @return <string>             The full path to the template file.
     */
    function getTemplate($template = NULL, $ext = '.php', $type = 'path') {
        if ($template == NULL)
            return false;
        $themeFile =  get_theme_root() . '/' . get_template() . '/' . $template . $ext;

        if (file_exists( $themeFile ) ) {
            if ($type == 'url') {
                $file = get_option('siteurl') . '/wp-content/themes/' . get_template() . '/' . $template . $ext;
            } else {
                $file = get_theme_root() . '/' . get_template() . '/' . $template . $ext;
            }
        }
        elseif ($type == 'url') {
            $file = $this->templatesURL . $template . $ext;
        }
        else {
            $file = $this->templatesPath . $template . $ext;
        }

        return $file;
    }

    public function event_registration() {
        global $current_user;
        get_currentuserinfo();
        $type = $_POST['type'];
        $id = $_POST['id'];
        $action = $_POST['click_action'];
        $event = get_post($id);

        switch ($action) {
            case 'delete':
                $registrations = get_user_meta($current_user->id, '_event_' . $type, true);

                if ($key = array_search($id, $registrations)) {
                    unset ($registrations[$key]);
                }

                $results = update_user_meta($current_user->ID, '_event_' . $type, $registrations);

                if ($results) {
                    $message = __('Your registration for ' . $event->post_title . ' has been canceled', 'calendar-press');
                    $status = 'Cancelled';
                } else {
                    $status = 'Error';
                    $message = __('Sorry, I was unable to remove your registrationr for ' . $event->post_title . '. Pleae try again later.' , 'calendar-press');
                }
                break;
            case 'move':
                /* Remove all registrations */
                foreach (array('signups','overflow') as $dtype) {
                    $registrations = get_user_meta($current_user->id, '_event_' . $dtype, true);

                    if (!is_array($registrations)) {
                        $registrations = array($registrations);
                    }

                    if ($key = array_search($id, $registrations)) {
                        unset ($registrations[$key]);
                    }
                    $results = update_user_meta($current_user->ID, '_event_' . $dtype, $registrations);
                }
                /* Add new registration */
                $registrations = get_user_meta($current_user->id, '_event_' . $type, true);
                $registrations[] = $id;
                $results = update_user_meta($current_user->ID, '_event_' . $type, $registrations);

                if ($results) {
                    $message = __('Your registration for ' . $event->post_title . ' has been moved to the ' . $this->options[$type.'-title'], 'calendar-press');
                    $status = 'Moved';
                } else {
                    $status = 'Error';
                    $message = __('Sorry, I was unable to remove your registrationr for ' . $event->post_title . '. Pleae try again later.' , 'calendar-press');
                }
                break;
            default:
                $registrations = get_user_meta($current_user->id, '_event_' . $type, true);
                if (!is_array($registrations)) {
                    $registrations = array($registrations);
                }
                if (in_array($id, $registrations)) {
                    $message = __('You are already registered on the ' . $this->options[$type . '-title'] . '  for the ' . $event->post_title . '.', 'calendar-press');
                    $status = 'Registered';
                } else {
                    $registrations[] = $id;
                    $results = update_user_meta($current_user->ID, '_event_' . $type, $registrations);
                    $message = __('You are now registered on the ' . $this->options[$type . '-title'] . ' for the ' . $event->post_title . '.', 'calendar-press');
                    $status = 'Registered';
                }
        }
        die('onSackSuccess("' . $status . '","' . $type . '", ' . $id . ', "' . $message . '")');
    }

    /**
     * Display the contributors list.
     *
     * @param <string> $type
     * @return <string>
     */
    public function showContributors($type = 'contributors') {
        $this->showFields = array('NAME', 'LOCATION' , 'COUNTRY');
        echo '<ul>';

        $xml_parser = xml_parser_create();
        xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
        xml_set_element_handler($xml_parser, array($this,"startElement"), array($this, "endElement") );
        xml_set_character_data_handler($xml_parser, array($this, "characterData") );

        if (!($fp = @fopen($this->xmlURL . $type . '.xml', "r"))) {
            /* translators: Message displayed when the contributors list cannot be accessed. */
            _e('There was an error getting the list. Try again later.', 'calendar-press');
            return;
        }

        while ($data = fread($fp, 4096)) {
            if (!xml_parse($xml_parser, $data, feof($fp))) {
                die(sprintf("XML error: %s at line %d",
                    xml_error_string(xml_get_error_code($xml_parser)),
                    xml_get_current_line_number($xml_parser)));
            }
        }

        xml_parser_free($xml_parser);
        echo '</ul>';
    }

    public function startElement($parser, $name, $attrs) {
        if ($name == 'NAME') {
            echo '<li class="rp-contributor">';
        } elseif ($name == 'ITEM') {
            /* translators: Used on the Contributors list to denote what a person contributed. */
            echo '<br><span class="rp_contributor_notes">' . __('Contributed: ', 'calendar-press');
        }

        if ($name == 'URL') {
            $this->makeLink = true;
        }
    }

    public function endElement($parser, $name) {
        if ($name == 'ITEM') {
            echo '</li>';
        }
        elseif ($name == 'ITEM') {
            echo '</span>';
        }
        elseif ( in_array($name, $this->showFields)) {
            echo ', ';
        }
    }

    public function characterData($parser, $data) {
        if ($this->makeLink) {
            echo '<a href="http://' . $data . '" target="_blank">' . $data . '</a>';
            $this->makeLink = false;
        } else {
            echo $data;
        }
    }
}

/* Instantiate the Plugin */
$calendarPressOBJ = new calendarPress;

/* Add Widgets */
//require_once('widgets/list-widget.php');
//require_once('widgets/category-widget.php');