<?php if(preg_match('#' . basename(__FILE__) . '#', $_SERVER['PHP_SELF'])) { die('You are not allowed to call this page directly.'); }

/**
 * initialize.php - Initialize the post types and taxonomies
 *
 * @package Calendar Press
 * @subpackage includes
 * @author GrandSlambert
 * @copyright 2009-2010
 * @access public
 */

class calendarPress_Init {

    public $options;

    public function __construct($options) {
        $this->options = $options;

        $this->pluginPath = WP_CONTENT_DIR . '/plugins/' . basename(dirname(dirname(__FILE__)));
        $this->pluginURL = get_option('siteurl') . '/wp-content/plugins/' . basename(dirname(dirname(__FILE__)));
        $this->templatesPath = WP_CONTENT_DIR . '/plugins/' . basename(dirname(dirname(__FILE__))) . '/templates/';
        $this->templatesURL = get_option('siteurl') . '/wp-content/plugins/calendar-press/templates/';

        add_action('init', array($this, 'create_post_type') );
        add_filter('manage_edit-events_columns', array(&$this, 'event_edit_columns'));
        add_action("manage_posts_custom_column",  array(&$this, 'event_custom_columns'));

        if ($this->options['use-thumbnails']) {
            //add_theme_support( 'post-thumbnails');
        }

        if ($this->options['use-categories']) {
            add_action('init', array($this, 'setup_categories') );
        }

        if ($this->options['use-tags']) {
            add_action('init', array($this, 'add_tags') );
        }

        if ($this->options['use-tags']) {
            register_taxonomy_for_object_type('post_tag', 'Events');
        }
    }

    public function create_post_type() {
        $page = get_page($this->options['calendar-page']);
        $labels = array(
            'name'              => __('Calendar Press', 'post type general name', 'calendar-press'),
            'singular_name'     => __('Event', 'post type singular name', 'calendar-press'),
            'add_new'           => __('Add New Event', 'Event', 'calendar-press'),
            'add_new_item'      => __('Add New Event', 'calendar-press'),
            'edit_item'         => __('Edit Event', 'calendar-press'),
            'edit'              => __('Edit', 'Event', 'calendar-press'),
            'new_item'          => __('New Event', 'calendar-press'),
            'view_item'         => __('View Event', 'calendar-press'),
            'search_items'      => __('Search Event', 'calendar-press'),
            'not_found'         =>  __('No Events found', 'calendar-press'),
            'not_found_in_trash'=> __('No Events found in Trash', 'calendar-press'),
            'view'              =>  __('View Event', 'calendar-press'),
            'parent_item_colon' => ''
        );
        $args = array(
            'labels'            => $labels,
            'public'            => true,
            'publicly_queryable'=> true,
            'show_ui'           => true,
            'query_var'         => true,
            'rewrite'           => array('slug'=> $page->post_name),
            'capability_type'   => 'post',
            'hierarchical'      => false,
            'menu_position'     => 5,
            'menu_icon'         => $this->options['menu-icon'],
            'supports'          => array('title','editor','author','excerpt'),
            'register_meta_box_cb' => array(&$this, 'init_metaboxes'),
        );

        if ($this->options['use-custom-fields']) {
            $args['supports'][] = 'custom-fields';
        }

        if ($this->options['use-thumbnails']) {
            $args['supports'][] = 'thumbnail';
        }

        if ($this->options['use-comments']) {
            $args['supports'][] = 'comments';
        }

        if ($this->options['use-trackbacks']) {
            $args['supports'][] = 'trackbacks';
        }

        if ($this->options['use-revisions']) {
            $args['supports'][] = 'revisions';
        }

        if ($this->options['use-post-tags']) {
            $args['taxonomies'][] = 'post_tag';
        }

        if ($this->options['use-post-categories']) {
            $args['taxonomies'][] = 'category';
        }

        register_post_type( $this->options['slug'] , $args);
    }

    public function event_edit_columns($columns) {
        $columns= array(
            'cb'        => '<input type="checkbox" />',
            'thumbnail' => 'Image',
            'title'     => 'Event Title',
            'intro'     => 'Introduction',
            'category'  => 'Categories',
            'featured'  => 'Featured',
            'author'    => 'Author'
        );

        if ($this->options['use-comments']) {
            $columns['comments'] = '<img src="' . get_option('siteurl') . '/wp-admin/images/comment-grey-bubble.png" alt="Comments">';
        }

        $columns['date'] = 'Date';


        return $columns;
    }

    public function event_custom_columns($column) {
        global $post;

        if ($post->post_type != $this->options['slug']) {
            return;
        }

        switch ($column) {
            case 'thumbnail':
                if (function_exists('has_post_thumbnail') && has_post_thumbnail()) {
                    the_post_thumbnail(array(50,50));
                }
                break;
            case 'intro':
                echo cp_inflector::trim_excerpt($post->post_excerpt, 25);
                break;
            case 'featured':
                if ( get_post_meta($post->ID, '_event_featured_value', true) ) {
                    _e('Yes', 'calendar-press');
                } else {
                    _e('No', 'calendar-press');
                }
                break;
            case 'category':
                echo get_the_term_list($post->ID, 'event-categories', '', ', ' , '');
                break;
        }
    }

    public function setup_categories() {
        /* Add Event Categories */
        $labels = array(
            'name'          => __('Event Categories', 'calendar-press'),
            'singular_name' => __('Category', 'calendar-press'),
            'search_items'  => __('Search By Category', 'calendar-press'),
            'popular_items' => __('Popular Categories', 'calendar-press'),
            'all_items'     => __('All Categories', 'calendar-press'),
            'parent_item'   => __('Parent Category', 'calendar-press'),
            'edit_item'     => __('Edit Category', 'calendar-press'),
            'update_item'   => __('Update Category', 'calendar-press'),
            'add_new_item'  => __('Add Category', 'calendar-press'),
            'new_item_name' => __('New Category', 'calendar-press')
        );

        register_taxonomy('event-categories', array('events'), array(
            'hierarchical'  => true,
            'label'         => __('Event Categories', 'calendar-press'),
            'labels'        => $labels,
            'public'        => true,
        ));
    }

    public function add_tags() {
        /* Add Tags Types */
        $labels = array(
            'name'          => __('Event Tags', 'recipe-press'),
            'singular_name' => __('Tag', 'recipe-press'),
            'search_items'  => __('Search By Tag', 'recipe-press'),
            'popular_items' => __('Popular Tags', 'recipe-press'),
            'all_items'     => __('All Tags', 'recipe-press'),
            'edit_item'     => __('Edit Tag', 'recipe-press'),
            'update_item'   => __('Update Tag', 'recipe-press'),
            'add_new_item'  => __('Add Tag', 'recipe-press'),
            'new_item_name' => __('New Tag', 'recipe-press')
        );

        register_taxonomy('event-tags', 'events', array(
            'hierarchical'  => false,
            'label'         => __('Tags', 'calendar-press'),
            'labels'        => $labels,
            'public'        => true,
        ));
    }

    function init_metaboxes() {
        add_meta_box('events_dates', __('Date and Time', 'calendar-press'), array(&$this, 'date_box'), 'events', 'side', 'high');
        add_meta_box('events_details', __('Details', 'calendar-press'), array(&$this, 'details_box'), 'events', 'side', 'high');

        if ($this->options['use-locations']) {
            add_meta_box('events_location', __('Event Location', 'calendar-press'), array(&$this, 'location_box'), 'events', 'advanced', 'high');
        }
    }

    function date_box() {
        global $post;
        // Use nonce for verification
        echo '<input type="hidden" name="dates_noncename" id="dates_noncename" value="' . wp_create_nonce( 'calendar_press_dates' ) . '" />';
        include ('date-box.php');
    }
    
    function details_box() {
        global $post;
        // Use nonce for verification
        echo '<input type="hidden" name="details_noncename" id="details_noncename" value="' . wp_create_nonce( 'calendar_press_details' ) . '" />';
        include ('details-box.php');
    }

    function location_box() {
        global $post;
        // Use nonce for verification
        echo '<input type="hidden" name="location_noncename" id="location_noncename" value="' . wp_create_nonce( 'calendar_press_location' ) . '" />';
        include ('locations-box.php');
    }
 }